package  
{
	import org.flixel.FlxSprite;
	import flash.geom.Vector3D;
	import org.flixel.plugin.photonstorm.FlxColor;
	import org.flixel.FlxG;
	/**
	 * ...
	 * @author Nathan McCoy
	 */
	public class Critter extends FlxSprite 
	{
		[Embed(source = "../lib/dash.mp3")]
		public static const DashSound:Class;
		[Embed(source = "../lib/midattack.mp3")]
		public static const AttackSound:Class;
		[Embed(source = "../lib/hit.mp3")]
		public static const HitSound:Class;
		[Embed(source = "../lib/die.mp3")]
		public static const DieSound:Class;
		[Embed(source = "../lib/lockon.mp3")]
		public static const LockSound:Class;
		
		
		
		public static const DASH_TIME:int = 18;
		public static const DASH_IMMUNITY:int = 12;
		public static const KNOCKBACK_TIME:int = 20;
		public static const SPAWN_TIME:int = 120;
		public var spawn_timer:int;
		
		//evolvable stats
		public var walk_speed:Number;
		public var attack_strength:Number;
		public var attack_speed:Number;
		public var attack_knockback:Number;
		public var attack_reach:Number;
		public var max_hp:Number;
		public var dash_range:Number;
		
		public var tint_r:int;
		public var tint_g:int;
		public var tint_b:int;
		
		public var radius:Number;
		
		public var BASE_WALK:Number = 100;
		public var BASE_POWER:Number = 20;
		public var BASE_COMBO:Number = 15;
		public var BASE_KNOCKBACK:Number = 200;
		public var BASE_REACH:Number = 20;
		public var BASE_HP:Number = 100;
		public var BASE_DASH:Number = 400;
		
		public var BASE_R:int = 192;
		public var BASE_G:int = 192;
		public var BASE_B:int = 192;
		public static const POUNCE_BONUS:Number = 2;
		
		public static const STAT_SCALING_FACTOR:Number = 21.0 / 20.0;
		
		private var _mutation:Mutation;
		
		public var movement_intent:Vector3D;
		public var facing_vector:Vector3D;
		public var dash_vector:Vector3D;
		public var dash_timer:int;
		
		public var knockback_timer:int;
		public var swing_timer:int;
		public var knockback_vector:Vector3D;
		
		public var key_attack:Boolean;
		public var key_attack_edge:Boolean;
		public var key_dash:Boolean;
		public var key_dash_edge:Boolean;
		public var attack_queued:Boolean;
		
		public var my_attack:Attack;
		
		public var native_color:uint;
		public var pouncemode:Boolean;
		
		public var my_lifedot:Lifedot;
		
		public function get fitness():Number
		{
			return -1;
		}
		
		public function get invulnerable():Boolean
		{
			return (DASH_TIME - dash_timer < DASH_IMMUNITY) || spawn_timer > 0;
		}
		
		public var hp:Number = 0;
		
		public function get unconfined():Boolean
		{
			return swing_timer <= 0 && knockback_timer <= 0 && dash_timer <= 0;
		}
		
		public function get ps():PlayState
		{
			return FlxG.state as PlayState;
		}
		
		public function hit(target:Critter):void
		{
			target.takeHit(this, attack_strength);
		}
		public function takeHit(attacker:Critter, damage:Number):void
		{
			var pouncemult:Number = 1;
			if (attacker.dash_timer > 0) pouncemult = POUNCE_BONUS;
			damage *= pouncemult;
			
			knockback_timer = KNOCKBACK_TIME;
			knockback_vector.x = x - attacker.x;
			knockback_vector.y = y - attacker.y;
			knockback_vector.normalize();
			knockback_vector.scaleBy(attacker.attack_knockback * pouncemult);
			hp -= damage;
			FlxG.play(HitSound);
			ps.damageParticle(x, y, damage, attacker == ps.player, ps.player.drive);
		}
		
		public function get mutation():Mutation
		{
			return _mutation;
		}
		public function set mutation(m:Mutation):void
		{
			_mutation = m;
			recalcStats();
		}
		public function recalcStats():void
		{
			walk_speed = BASE_WALK * Math.pow(STAT_SCALING_FACTOR, _mutation.data[Mutation.WALK_SPEED]);
			attack_strength = BASE_POWER * Math.pow(STAT_SCALING_FACTOR, _mutation.data[Mutation.ATTACK_STRENGTH]);
			attack_speed = BASE_COMBO * Math.pow(STAT_SCALING_FACTOR, -_mutation.data[Mutation.ATTACK_SPEED]);
			attack_knockback = BASE_KNOCKBACK * Math.pow(STAT_SCALING_FACTOR, _mutation.data[Mutation.ATTACK_KNOCKBACK]);
			attack_reach = BASE_REACH * Math.pow(STAT_SCALING_FACTOR, _mutation.data[Mutation.ATTACK_REACH]);
			max_hp = BASE_HP * Math.pow(STAT_SCALING_FACTOR, _mutation.data[Mutation.MAX_HP]);
			dash_range = BASE_DASH * Math.pow(STAT_SCALING_FACTOR, _mutation.data[Mutation.DASH_RANGE]);
			
			tint_r = Math.max(Math.min(BASE_R + _mutation.data[Mutation.TINT_R] * 15, 255),0);
			tint_g = Math.max(Math.min(BASE_G + _mutation.data[Mutation.TINT_G] * 15, 255),0);
			tint_b = Math.max(Math.min(BASE_B + _mutation.data[Mutation.TINT_B] * 15, 255),0);
			
			native_color = FlxColor.getColor24(tint_r, tint_g, tint_b);
			color = native_color;
			hp = max_hp;
		}
		
		public function Critter(X:Number=0, Y:Number=0, SimpleGraphic:Class=null) 
		{
			super(X, Y, SimpleGraphic);
			movement_intent = new Vector3D();
			dash_vector = new Vector3D();
			facing_vector = new Vector3D(1, 0);
			knockback_vector = new Vector3D();
			swing_timer = knockback_timer = dash_timer = 0;
			mutation = new Mutation();
			attack_queued = false;
			my_attack = new Attack(this);
			if (ps.attacks) ps.attacks.add(my_attack);
			hp = max_hp;
			spawn_timer = SPAWN_TIME;
			alpha = 0;
			pouncemode = false;
		}
		
		public function initiateAttack():void
		{
			swing_timer = attack_speed;
			attack_queued = false;
			FlxG.play(AttackSound);
			velocity.x = 0;
			velocity.y = 0;
			my_attack.x = x;
			my_attack.y = y;
			my_attack.exists = true;
		}
		
		public function die():void
		{
			exists = false;
			ps.attacks.remove(my_attack, true);
			FlxG.play(DieSound);
			for (var ix:int = -1; ix <= 1; ix++)
				for (var iy:int = -1; iy <= 1; iy++)
				{
					var dp:DeathParticle = ps.recycle(DeathParticle) as DeathParticle;
					dp.init(x, y, ix * 200, iy * 200);
				}
		}
		
		public override function update():void
		{
			color = native_color;
			spawn_timer--;
			if (spawn_timer > 0)
			{
				spawn_timer--;
				alpha = (SPAWN_TIME-spawn_timer) * 1.0 / SPAWN_TIME;
				return;
			}
			
			
			
			if (key_attack_edge) attack_queued = true;
			if (attack_queued && dash_timer <= 0 && swing_timer <= 0 && knockback_timer <= 0)
			{
				if (movement_intent.x != 0 || movement_intent.y != 0)
				{
					facing_vector.x = movement_intent.x;
					facing_vector.y = movement_intent.y;
					facing_vector.normalize();
				}
				initiateAttack();
			}
			
			
			if (key_dash_edge && dash_timer <= 0)
			{
				dash_vector.x = movement_intent.x;
				dash_vector.y = movement_intent.y;
				
				if (dash_vector.x == 0 && dash_vector.y == 0 && pouncemode)
				{
					if (ps.reticle.visible)
					{
						dash_vector.x = ps.reticle.x - x;
						dash_vector.y = ps.reticle.y - y;
						
					}
					else
					{
						dash_vector.x = facing_vector.x;
						dash_vector.y = facing_vector.y;
					}
				}
				
				if (!(dash_vector.x == 0 && dash_vector.y == 0))
				{
					dash_vector.normalize();
					facing_vector.x = dash_vector.x;
					facing_vector.y = dash_vector.y;
					dash_vector.scaleBy(dash_range);
					dash_timer = DASH_TIME;
					FlxG.play(DashSound);
					swing_timer = 0;
					knockback_timer = 0;
					if (pouncemode) initiateAttack();
				}
			}
			if (knockback_timer > 0)
			{
				var kbleft:Number = 1.0 * knockback_timer / KNOCKBACK_TIME;
				velocity.x = knockback_vector.x * kbleft;
				velocity.y = knockback_vector.y * kbleft;
			}
			if (dash_timer > 0)
			{
				var dash_remaining:Number = dash_timer * 1.0 / DASH_TIME;
				velocity.x = dash_vector.x * dash_remaining;
				velocity.y = dash_vector.y * dash_remaining;
				if (DASH_TIME - dash_timer < DASH_IMMUNITY)
				{
					color = FlxColor.interpolateColor(0xFFFFFF,color,  DASH_IMMUNITY, DASH_TIME-dash_timer);
				}
				if (dash_timer % 4 == 0)
				{
					var ech:EchoParticle = ps.particles.recycle(EchoParticle) as EchoParticle;
					ech.init(this);
				}
			}
			
			if (dash_timer <= 0 && knockback_timer <= 0 && swing_timer <= 0)
			{
				var walk_vector:Vector3D = new Vector3D();
				walk_vector.x = movement_intent.x;
				walk_vector.y = movement_intent.y;
				walk_vector.normalize();
				walk_vector.scaleBy(walk_speed);
				velocity.x = walk_vector.x;
				velocity.y = walk_vector.y;
				
				if (walk_vector.lengthSquared != 0)
				{
					facing_vector.x = walk_vector.x;
					facing_vector.y = walk_vector.y;
					facing_vector.normalize();
				}
			}
			
			if (knockback_timer <= 0 && hp <= 0) die();
			
			if (x < radius) 
			{
				x = radius;
				knockback_vector.x *= -1;
			}
			if (x > FlxG.width - radius)
			{
				x = FlxG.width - radius;
				knockback_vector.x *= -1;
			}
			if (y < radius) 
			{
				y = radius;
				knockback_vector.y *= -1;
			}
			if (y > FlxG.height - radius)
			{
				y = FlxG.height - radius;
				knockback_vector.y *= -1;
			}
			dash_timer--;
			knockback_timer--;
			swing_timer--;
		}
		
		
	}

}