package  
{
	/**
	 * ...
	 * @author Nathan McCoy
	 */
	public class Mutation 
	{
		public static const WALK_SPEED:int = 0;
		public static const ATTACK_STRENGTH:int = 1;
		public static const ATTACK_SPEED:int = 2;
		public static const ATTACK_KNOCKBACK:int = 3;
		public static const ATTACK_REACH:int = 4;
		public static const MAX_HP:int = 5;
		public static const DASH_RANGE:int = 6;
		
		public static const TINT_R:int = 7;
		public static const TINT_G:int = 8;
		public static const TINT_B:int = 9;
		public static const AI_AGGRESSION:int = 10;
		
		public var data:Vector.<int>;
		
		private static const VISIBLE_UP_TO:int = DASH_RANGE;
		public static const TRAIT_NAMES:Vector.<String> = new < String > ["Move", "Power", "Combo", "Knockback", "Reach", "HP", "Dash"];
		private static const OBVIOUSNESS_THRESHOLD:int = 0;
		//private static const PROMINENT_TRAITS:int = 3;
		
		private static const UP_SIGN:String = "\u2020";
		private static const DOWN_SIGN:String = "\u2021";
		
		public var mass:Number;
		public var quality:Number;
		
		public function Mutation(dice:Number = 0, luck:Number = 0) 
		{
			data = new Vector.<int>(11, true);
			randomize(dice, luck);
			mass = dice;
			quality = luck;
		}
		public function clone():Mutation
		{
			var m:Mutation = new Mutation();
			m.data = data.slice();
			return m;
		}
		
		public function get overview():String
		{
			var out:String = "";
			for (var i:int = 0; i <= VISIBLE_UP_TO; i++)
			{
				out += TRAIT_NAMES[i] + ": " + data[i] + "\n";
			}
			return out;
		}
		
		public function get description():String
		{
			var desc:String = "";
			var salient:int = 0;
			var others:int = 0;
			
			for (var i:int = 0; i <= VISIBLE_UP_TO; i++)
			{
				salient = Math.max(salient, Math.abs(data[i]));
			}
			salient -= OBVIOUSNESS_THRESHOLD;
			if (salient < 1) salient = 1;
			
			for (i = 0; i <= VISIBLE_UP_TO; i++)
			{
				if (Math.abs(data[i]) >= salient)
				{
					desc += TRAIT_NAMES[i]+" ";
					var sign:String;
					if (data[i] > 0) 
						sign = UP_SIGN; 
					else 
						sign = DOWN_SIGN;
					for (var j:int = 0; j < Math.abs(data[i]); j++)
					   desc += sign;
					desc += "\n";
				}
				else if (data[i] != 0) others++;
			}
			if (desc == "") desc = "(none)";
			desc = "Size: " + mass + "  Quality: " + int(quality * 10) +"\n\nProminent Traits:\n" + desc +"\n(other effects: "+others+")";
			return desc;
		}
		
		public static function LuckyF(rerolls:Number = 0):int
		{
			var result:int = Math.floor(Math.random() * 3) - 1;
			if (rerolls > 0)
			{
				for (var i:int = 0; i < Math.floor(rerolls); i++) //integer rerolls
				{
					result = Math.max(LuckyF(), result); //take best die
				}
				if (Math.random() < rerolls - int(rerolls)) //maybe a fractional reroll
					result = Math.max(LuckyF(), result);
			}
			
			if (rerolls < 0)
			{
				rerolls = -rerolls;
				for (i = 0; i < Math.floor(rerolls); i++) //integer rerolls
				{
					result = Math.min(LuckyF(), result); //take worst die
				}
				if (Math.random() < rerolls - int(rerolls)) //maybe a fractional reroll
					result = Math.min(LuckyF(), result);
			}
			return result;
		}
		
		public static function XdF(dice:Number = 1, luck:Number = 0):int
		{
			var total:int = 0;
			for (var i:int = 0; i < Math.floor(dice); i++)
			{
				total += LuckyF(luck);
			}
			//roll a fractional die, maybe
			if (Math.random() < dice - int(dice)) total += LuckyF(luck);
			
			return total;
		}
		public function randomize(dice:Number = 0, luck:Number = 0):void
		{
			for (var i:int = 0; i < data.length; i++)
			{
				data[i] = XdF(dice, luck);
			}
		}
		public function absorb(other:Mutation):Mutation
		{
			for (var i:int = 0; i < data.length; i++)
			{
				data[i] += other.data[i];
				quality += other.quality;
				mass += other.mass;
			}
			return this;
		}
		
	}

}