package
{

	import org.flixel.*;
	import org.flixel.plugin.photonstorm.FlxColor;
	import flash.display.StageQuality;
	
	public class PlayState extends FlxState
	{
		
		[Embed(source = "../lib/bg.png")]
		public static const BackgroundGraphic:Class;
		[Embed(source = "../lib/music.mp3")]
		public static const BackgroundMusic:Class;
		[Embed(source = "../lib/reticle.png")]
		public static const ReticleGraphic:Class;
		
		public var critters:FlxGroup;
		public var attacks:FlxGroup;
		public var particles:FlxGroup;
		
		public var score:int;
		public var overdrive_combo:int;
		public var peak_percent:int;
		public var bonus:int;
		
		public var player:PlayerCritter;
		public var xp_text:FlxText;
		public var drive_text:FlxText;
		public var frame_spinner:int;
		public var driveMeter:FlxSprite;
		public var dramatimer:int;
		public var mutationqueue:Array;
		
		public var reticle:FlxSprite;
		
		public static const BASE_BG:uint = 0xFF204830;
		
		public var mutation_overlay:FlxGroup;
		public var mutation_offer:FlxText;
		public var mutation_title:FlxText;
		public var stat_view:FlxText;
		public var score_text:FlxText;
		public var fancy_banner:FlxText;
		public var newgamenotice:FlxText;
		
		
		public function damageParticle(X:Number, Y:Number, damage:Number, player:Boolean = false, drive:Number = 0):DamageParticle
		{
			var dp:DamageParticle = particles.recycle(DamageParticle) as DamageParticle;
			dp.init(X, Y, damage, player, drive);
			return dp;
		}
		
		override public function create():void
		{
			FlxG.mouse.hide();
			FlxG.bgColor = BASE_BG;
			score = 0;
			overdrive_combo = 0;
			peak_percent = 0;
			bonus = 0;
			
			FlxG.stage.quality = StageQuality.LOW;
			
			mutationqueue = new Array();
			mutationqueue.push(new Mutation(4, 0.2));
			
			//add(new FlxSprite(0, 0, BackgroundGraphic));
			
			critters = new FlxGroup();
			attacks = new FlxGroup();
			particles = new FlxGroup();
						
			add(attacks);
			
			player = new PlayerCritter(FlxG.width / 2, FlxG.height / 2);
			
			critters.add(player);
			critters.add(new Foe());
			
			add(critters);
			add(new DriveAura());
			
			xp_text = new FlxText(2, 10, FlxG.width);
			xp_text.setFormat("FivePixel", 8, 0x00DDFF, "left");
			drive_text = new FlxText(2, 3, FlxG.width);
			drive_text.setFormat("FivePixel", 8, 0x00DDFF, "left");
			frame_spinner = 0;
			
			add(particles);
			
			reticle = new FlxSprite(0, 0, ReticleGraphic);
			reticle.width = reticle.height = 0;
			reticle.centerOffsets();
			reticle.visible = false;
			add(reticle);
			
			add(xp_text);
			add(drive_text);
			driveMeter = new FlxSprite(4, 2);
			driveMeter.makeGraphic(FlxG.width, 2, 0x0, false);
			add(driveMeter);
			score_text = new FlxText(2, 3, FlxG.width - 6);
			score_text.setFormat("FivePixel", 8, 0x22EE00, "right");
			add(score_text);
			
			fancy_banner = new FlxText(0, FlxG.height / 3, FlxG.width, "Overdrive");
			fancy_banner.setFormat("LargeNine", 27, 0xFFFFFF, "center");
			fancy_banner.alpha = 0;
			add(fancy_banner);
			
			
			add(new Lifedot(player));
			add(new Lifedot(player));
			add(new Lifedot(player));
			add(new Lifedot(player));
			
			
			mutation_overlay = new FlxGroup();
			mutation_overlay.add(new FlxSprite().makeGraphic(FlxG.width, FlxG.height, 0x60000000));
			mutation_overlay.add(new FlxText(0, FlxG.height * 0.8, FlxG.width, "X: Reject    C: Accept").setFormat("PressStartK", 8, 0xFFFFFF, "center"));
			mutation_overlay.add(mutation_title = new FlxText(0, FlxG.height * 0.2, FlxG.width, "Initial Mutation").setFormat("PressStartK", 8, 0xFFFFFF, "center"));
			mutation_offer = new FlxText(FlxG.width / 4, FlxG.height / 3-8, 200).setFormat("PressStartK");
			mutation_overlay.add(mutation_offer);
			
			add(mutation_overlay);
			
			stat_view = new FlxText(2, FlxG.height - 59, 100).setFormat("FivePixel");
			stat_view.alpha = 0.5;
			add(stat_view);
			
			newgamenotice = new FlxText(0, FlxG.height * 3 / 4, FlxG.width, "press C to play again");
			newgamenotice.setFormat("PressStartK", 8, 0xFFFFFF, "center");
			newgamenotice.alpha = 0.5;
			newgamenotice.visible = false;
			
			add(newgamenotice);
			
			
			dramatimer = 0;
			FlxG.playMusic(BackgroundMusic);
		}
		public function get suspend():Boolean
		{
			if (dramatimer > 0) return true;
			if (mutationqueue.length > 0 && !player.overdrive) return true;
			if (!player.exists) return true;
			return false;
		}
		
		public function drivecolor(driveLevel:Number):int
		{
				
			if (player.overdrive)
			{
				return FlxColor.HSVtoRGB((frame_spinner * 10) % 360, 0.3, 1);
			}
			if (driveLevel > 1) return 0xffffff;
			
			var flicker:Number = 0;
			if (driveLevel > 0.5 && (frame_spinner % 6) < 3) flicker = (driveLevel - 0.5) * 2;
			
			return(FlxColor.getColor24(255, 128 + 127 * driveLevel, 255 * flicker));
		}
		
		override public function update():void
		{
			if (suspend)
			{
				critters.active = false;
				attacks.active = false;
				//particles.active = false;
				
				if (mutationqueue.length > 0 && dramatimer <= 0) 
				{
					mutation_overlay.visible = true;
					var mut:Mutation = mutationqueue[0] as Mutation;
					mutation_offer.text = mut.description;
					if (FlxG.keys.justPressed("X"))
					{
						if (player.mutation.mass == 0) mutationqueue.push(new Mutation(4, 0.2));
						mutationqueue.splice(0,1);
					}
					else if (FlxG.keys.justPressed("C"))
					{
						mutation_title.text = "Additional Mutation";
						player.mutation.absorb(mut);
						player.recalcStats();
						stat_view.text = player.mutation.overview;
						mutationqueue.splice(0,1);
					}
				}
			}
			else
			{
				critters.active = true;
				attacks.active = true;
				particles.active = true;
				mutation_overlay.visible = false;
			}
			if (dramatimer > 0) dramatimer--;
			super.update();
			frame_spinner++;
			xp_text.text = "XP: " + Math.floor(player.xp);
			drive_text.text = "Drive: " + Math.floor(player.drive * 100) + "%";
			drive_text.setFormat("FivePixel", 8, drivecolor(player.drive));
			
			if (player.overdrive) FlxG.bgColor = FlxColor.interpolateColor(BASE_BG, 0x0, 3, 1);
			else FlxG.bgColor = BASE_BG;
			
			driveMeter.fill(0x0);
			driveMeter.drawLine(0, 1, 100, 1, FlxColor.interpolateColor(0x000000, drivecolor(player.drive), 4, 1), 3);
			driveMeter.drawLine(0, 1, player.drive * 100, 1, drivecolor(player.drive), 3);
			
			if (critters.countLiving() < 4 && player.exists) 
			{
				critters.sort("fitness", DESCENDING);
				var fittest:Critter = critters.getFirstExtant() as Critter;
				var newfoe:Critter = new Foe(Math.random() * FlxG.width, Math.random() * FlxG.height);
				newfoe.mutation = new Mutation(2, 0.3).absorb(fittest.mutation);
				critters.add(newfoe);
			}
			
			score_text.text = "Score: " + score;
			fancy_banner.alpha -= 0.015;
			fancy_banner.color = FlxColor.HSVtoRGB((frame_spinner * 20) % 360, 0.5, 1);
			//while (particles.remove(particles.getFirstAvailable(), true)){};
			
			if (player.closest_target != null)
			{
				reticle.visible = true;
				reticle.x = player.closest_target.x;
				reticle.y = player.closest_target.y;
				reticle.color = FlxColor.getColor24(255, 255 * ((frame_spinner / 4) % 2), ((frame_spinner / 2) % 2));
			}
			else reticle.visible = false;
			
			if (!player.exists && dramatimer <= 0) newgamenotice.visible = true;
			if (!player.exists && dramatimer <= 0 && FlxG.keys.justPressed("C")) FlxG.resetState();
		}
	}
}

