package  
{
	import org.flixel.FlxG;
	import org.flixel.plugin.photonstorm.FlxColor;
	/**
	 * ...
	 * @author Nathan McCoy
	 */
	
	public class PlayerCritter extends Critter 
	{
		[Embed(source = "../lib/dot2.png")]
		public static const DotGraphic:Class;
		[Embed(source = "../lib/overdrive.mp3")]
		public static const OverdriveSound:Class;
		[Embed(source = "../lib/overdrive-end.mp3")]
		public static const OverdriveEndSound:Class;
		
		
		
		public var xp:Number;
		public var drive:Number;
		public const DRIVE_DRAIN_RATE:Number = 0.002;
		public const XP_DRIVE_RATIO:Number = 0.002;
		public const XP_DRAIN_RATE:Number = 0.15;
		public var overdrive:Boolean;
		public var lock_timer:int;
		public const LOCK_TIME:int = 30;
		public const OVERDRIVE_LOCK_TIME:int = 5;
		
		public function PlayerCritter(X:Number = 0, Y:Number = 0, SimpleGraphic:Class = null) 
		{
			super(X, Y, SimpleGraphic);
			loadGraphic(DotGraphic);
			radius = 8;
			width = 0; height = 0; centerOffsets();
			
			xp = 50;
			drive = 0;
			BASE_HP = 200;
			BASE_REACH = 26;
			
			recalcStats();
			hp = max_hp;
			//mutation = new Mutation(4, 0);
		}
		public override function hit(target:Critter):void
		{
			xp++;
			drive += XP_DRIVE_RATIO * xp;
			if (dash_timer > 0) 
			{
				xp++;
				drive += XP_DRIVE_RATIO * xp / 2; //extra drive and xp for a pounce
			}
			target.takeHit(this, attack_strength * (1.0 + drive));
		}
		
		public override function initiateAttack():void
		{
			super.initiateAttack();			
		}
		
		override public function die():void 
		{
			super.die();
			ps.fancy_banner.text = "GAME OVER";
			ps.fancy_banner.alpha = 1;
			ps.dramatimer = 180;
			FlxG.music.stop();
			
		}
		public var closest_target:Foe = null;
		public override function update():void
		{
			movement_intent.x = 0;
			movement_intent.y = 0;
			
			if (FlxG.keys.DOWN) movement_intent.y += 1;
			if (FlxG.keys.UP) movement_intent.y -= 1;
			if (FlxG.keys.LEFT) movement_intent.x -= 1;
			if (FlxG.keys.RIGHT) movement_intent.x += 1;
			
			key_dash_edge = FlxG.keys.justPressed("X");
			key_dash = FlxG.keys.X;
			key_attack_edge = FlxG.keys.justPressed("C");
			key_attack = FlxG.keys.C;
			
			if (drive > 1 && !overdrive && unconfined) 
			{
				overdrive = true;
				attack_speed *= 0.7;
				walk_speed *= 1.7;
				FlxG.play(OverdriveSound);
				hp = max_hp;
				ps.dramatimer = 90;
				FlxG.flash(0x44FFFFFF, 0.6);
				ps.fancy_banner.text = "OVERDRIVE";
				ps.fancy_banner.alpha = 1;
			}
			
			if (overdrive) 
			{
				xp = Math.max(0, xp - XP_DRAIN_RATE);
				ps.peak_percent = Math.max(ps.peak_percent, int(drive * 100));
				
			}
			
			drive = Math.max(drive - DRIVE_DRAIN_RATE, 0);
			if (drive == 0 && overdrive) 
			{
				overdrive = false;
				recalcStats();
				ps.bonus = ps.overdrive_combo * (100 + ps.peak_percent);
				ps.overdrive_combo = 0;
				ps.peak_percent = 0;
				ps.fancy_banner.text = "+" + ps.bonus;
				ps.score += ps.bonus;
				ps.fancy_banner.alpha = 1;
				FlxG.play(OverdriveEndSound);
				
				ps.dramatimer = 150;
				FlxG.flash(0x44FFFFFF, 0.6);
			}
			
			
			closest_target = null;
			if (pouncemode)
			{
				var distance:Number = 1000 * 1000;
				for (var i:int = 0; i < ps.critters.length; i++)
				{
					if (ps.critters.members[i] != this && ps.critters.members[i].exists)
					{
						var candidate:Foe = ps.critters.members[i] as Foe;
						var testDist:Number = (candidate.x - x) * (candidate.x - x) + (candidate.y - y) * (candidate.y - y);
						if (testDist < distance)
						{
							closest_target = candidate;
							distance = testDist;
						}
					}
				}
			}
			
			
			super.update();
			
			
			
			if (key_attack && swing_timer <= 0 && dash_timer <= 0 && knockback_timer <= 0)
			{	
				lock_timer++;
				if ((lock_timer >= LOCK_TIME || (overdrive && lock_timer >= OVERDRIVE_LOCK_TIME)) && !pouncemode) 
				{
					FlxG.play(LockSound);
					pouncemode = true;
				}
			}
			else 
			{
				lock_timer = 0;
				pouncemode = false;
			}
			
			if (overdrive)
			{
				color = FlxColor.interpolateColor(color, ps.drivecolor(1), 2, 1);
			}
			if (pouncemode && ps.frame_spinner % 4 < 2)
			{
				color = FlxColor.interpolateColor(color, 0xFFFFFF, 2, 1);
			}
			
		}
		override public function get invulnerable():Boolean
		{
			return super.invulnerable || overdrive;
		}
		override public function draw():void 
		{
			super.draw();
		}
		
	}

}